package body Colas_De_Prioridad is
	----------------------------
	-- Operaciones auxiliares --
	----------------------------

	-- Intercambiar intercambia los valores de sus parmetros
	procedure Intercambiar(X,Y: in out TElemento) is
		Aux: TElemento;
	begin
		Aux := X;
		X   := Y;
 		Y   := Aux;
	end Intercambiar;

	-- Hundir hace descender un elemento por un montculo hasta que
	-- alcance la posicin que le corresponde segn su prioridad
	procedure Hundir(M: in out Montculo; Tamao,Pos: in Natural) is
	begin	
		if (2 * Pos <= Tamao) then -- Tiene hijos
			if (2 * Pos + 1 <= Tamao) and then
				(MayorPrioridad(M(2 * Pos + 1),M(2 * Pos))) then
				-- Tiene ambos hijos y el derecho es el de mayor prioridad
				if mayorPrioridad(M(2 * Pos + 1),M(Pos)) then
					-- La prioridad del actual es menor que la de su hijo
					-- derecho
					Intercambiar(M(Pos),M(2 * Pos + 1));
					Hundir(M,Tamao,2 * Pos + 1);
				end if;
			else
				-- El hijo izquierdo es el nico hijo o el derecho
				-- no es ms prioritario
				if mayorPrioridad(M(2 * Pos),M(Pos)) then
					-- La prioridad del actual es menor que el de su hijo
					-- izquierdo
					Intercambiar(M(Pos),M(2 * Pos));
					Hundir(M,Tamao,2 * Pos);
				end if;
			end if;
		end if;
	end Hundir;

	-- Flotar hace ascender un elemento por un montculo hasta que
	-- alcance la posicin que le corresponde segn su prioridad
	procedure Flotar(M: in out Montculo; Pos: in Natural) is
	begin
		-- Si el elemento en Pos no es la raz, se compara con su padre
		-- para ver quin tiene mayor prioridad
		if (Pos > 1) and then MayorPrioridad(M(Pos),M(Pos / 2)) then
			-- Se intercambia el elemento en Pos con su padre de
			-- menor prioridad
			Intercambiar(M(Pos),M(Pos / 2));
			-- Se repite la operacin hasta encontrar un padre de
			-- mayor prioridad o llegar a la raz
			Flotar(M,Pos / 2);
		end if;
	end Flotar;

	----------------------------------
	-- Operaciones pblicas del tad --
	----------------------------------

	-- Insertar aade un elemento al final del montculo y llama
	-- a Flotar para hacerlo llegar a su posicin
	procedure Insertar(C: in out Cola; E: in TElemento) is
	begin
		C.Tamao              := C.Tamao + 1;
		C.Elementos(C.Tamao) := E;
		Flotar(C.Elementos,C.Tamao);
	end Insertar;
	
	-- Extraer quita el ltimo elemento, lo pone en lugar de la raz
	-- y llama a Hundir para hacerlo llegar a su posicin
	procedure Extraer(C: in out Cola) is
	begin
		C.Elementos(1) := C.Elementos(C.Tamao);
		C.Tamao       := C.Tamao - 1;
		Hundir(C.Elementos,C.Tamao,1);
	end Extraer;

	-- Devuelve el elemento de mayor prioridad
	function Examinar(C: in Cola) return TElemento is
	begin
		if C.Tamao = 0 then
			raise Constraint_Error;
		else
			return C.Elementos(1);
		end if;
	end Examinar;

	-- Vaca la cola de prioridad
	procedure Vaciar(C: in out Cola) is
	begin
		C.Tamao := 0;
	end Vaciar;

	-- Devuelve el nmero de elementos de la cola de prioridad
	function Tamao(C: in Cola) return Natural is
	begin
		return C.Tamao;
	end;
end Colas_De_Prioridad;
