with Unchecked_Deallocation;

package body Contenedores.Tries is
	procedure Liberar is new Unchecked_Deallocation(Nodo_Trie, Rbol_Trie);

	Procedure Insertar(
			T: in out Rbol_Trie;
			P: in Natural; -- Porcin de la clave a considerar
			               -- en este nivel
			E: in TElemento
			) is
		Clave_E: Clave_Completa := Clave(E);
		Nuevo  : Rbol_Trie; 
	begin
		if T = null then
			-- No hay nada con que comparar, se crea
			-- un nodo de informacin con el elemento
			T := new Nodo_Trie(Informacin);
			T.Info := E;
		else
			if T.Tipo = Desviacin then
				if P > Clave_E'Length then
					-- Se acab la clave, el elemento se inserta
					-- en el enlace final
 			    	Insertar(T.Final, P + 1, E);
				else
					-- Contina la insercin en el subrbol indicado
					-- por el dgito de la clave que ocupa la posicin P
					Insertar(T.Enlaces(Clave_E(P)), P + 1, e);
				end if;
			else
				if Clave(T.Info) /= Clave_E then
					-- Se crea un nodo desviacin para separar la
					-- clave existente y la nueva
					Nuevo := new Nodo_trie(Desviacin);
					if P > Clave(T.Info)'Length then
						Nuevo.Final := T;
					else
						Nuevo.Enlaces(Clave(T.Info)(P)) := T;
					end if;
					T := Nuevo;
					Insertar(T, P, E);
				end if;
			end if;
		end if;
	end Insertar;

	Procedure Extraer(
			T: in out Rbol_Trie;
			P: in Natural; -- Porcin de la clave a considerar
			               -- en este nivel
			E: in TElemento
			) is
		Clave_E: Clave_Completa  := Clave(E);
		J      : natural  := 0;
		No_Nulo: Rbol_Trie;
	begin
		if T /= null then
			if T.Tipo = Desviacin then
				if P <= Clave_E'Length then
					-- Contina la Extraccin en el subrbol indicado
					-- por el dgito de la clave que ocupa la posicin P
					Extraer(T.Enlaces(Clave_E(P)), P + 1, E);
				else
					-- Se acab la clave, el elemento slo puede estar
					-- en el enlace final
					Extraer(T.Final, P + 1, E);
				end if;

				-- Se mira cuantos hijos tiene T	
				for I in Parte_clave loop
					if  T.Enlaces(I) /= null then
						J := J + 1;
						No_nulo := T.Enlaces(I);
					end if;
					exit when J = 2;
				end loop;
				
				if T.Final /= null then
					J := J + 1;
					No_nulo := T.Final;
				end if;

				-- Si este nodo no tiene hijos, o tiene slo uno de
			 	-- tipo Informacin, se libera
				if j < 2 then
					if (J = 0) or else (No_Nulo.Tipo = Informacin) then
						Liberar(T);
						T := No_Nulo;
					end if;
				end if;
			else
				-- Si el elemento contenido en este nodo es el
			 	-- buscado, se elimina el nodo
				if Clave(T.Info) = Clave_E then
					Liberar(T);
				end if;
			end if;
		end if;
	end Extraer;

	function Buscar(
			T: in Rbol_Trie;
			P: in Natural; -- Porcin de la clave a considerar
			               -- en este nivel
			E: in TElemento
			) return Rbol_Trie is
		Clave_E: Clave_completa := Clave(E);
	begin
		if T /= null then
			if T.Tipo = Desviacin then
				if P > Clave_E'Length then
					-- Se acab la clave
 			    	return Buscar(T.Final, P + 1, E);
				else
					-- Contina la bsqueda en el subrbol indicado
					-- por el dgito de la clave que ocupa la posicin P
					return Buscar(T.Enlaces(Clave_E(P)), P + 1, E);
				end if;
			else
				if Clave(T.Info) = Clave_E then
					-- Se encontr la clave
					return T;
				end if;
			end if;
		end if;

		-- La clave no se encuentra
		return null;
	end Buscar;

	procedure Vaciar(T: in out rbol_Trie) is
	begin
		if T /= null then
			if T.Tipo = Desviacin then
				for I in Parte_clave loop
					Vaciar(T.Enlaces(I));
				end loop;
				Vaciar(T.Final);
			end if;
			Liberar(T);
		end if;
	end Vaciar;

	function Tamao(T: in rbol_Trie) return natural is
		Num_elementos: natural := 0;
	begin
		if T /= null then
			if T.Tipo = Desviacin then
				for i in Parte_clave loop
					Num_elementos := Num_elementos + Tamao(T.Enlaces(I));
				end loop;
				Num_elementos := Num_elementos + Tamao(T.Final);
			else
				Num_elementos := 1;
			end if;
		end if;
		return Num_elementos;
	end Tamao;
			

	-------------------------------
	-- Operaciones constructoras --
	-------------------------------	
	procedure Insertar(C: in out Trie; E: in TElemento) is
	begin
		Insertar(C.Elementos, 1, E);
	end Insertar;

	procedure Extraer(C: in out Trie; E: in TElemento) is
	begin
		Extraer(C.Elementos, 1, E);
	end Extraer;

	procedure Vaciar(C: in out Trie) is
	begin
		Vaciar(C.Elementos);
	end Vaciar;

	------------------------------
	-- Operaciones observadoras --
	------------------------------
	function Est(C: in Trie; E: in TElemento) return boolean is
	begin
		return Buscar(C.Elementos, 1, E) /= null;
	end Est;

	function Tamao(C: in Trie) return Natural is
	begin
		return Tamao(C.Elementos);
	end Tamao;
end Contenedores.Tries;
