generic
	type TElemento is private;
package rbol_Bin is
	type rbol is private;
	-- La asignacin genera un alias!
	-- La igualdad evala si dos variables son alias del mismo rbol!

	Vaco : constant rbol;
	-- Representa el rbol Vaco

	procedure Vaciar(T: in out rbol);
	-- Modifica T para representar un rbol Vaco. El valor anterior de T 
	-- es destruido de forma controlada

	function Crear(E: TElemento; T1, T2: rbol) return rbol;
	-- Crea un nuevo rbol, usando E como raz y T1 y T2 como subrboles
	-- izquierdo y derecho, respectivamente. No realiza copia de T1 y T2,
	-- sino que los usa directamente

	function Modificar(T: rbol; E: TElemento; T1, T2: rbol) return rbol;
	-- Modifica el rbol T cambiando su raz y sus subrboles izquierdo y 
	-- derecho por E, T1 y T2, respectivamente.
	-- No realiza copia de T1 ni de T2 sino que los usa directamente.
	-- Tampoco destruye los anteriores subrboles izquierdo y derecho 
	-- de T, para permitir su reutilizacin.
	-- T no puede ser un rbol vaco

	function Es_Vaco(T: rbol) return Boolean;
	-- Devuelve true si T es un rbol vaco y false en otro caso

	function Raz(T: rbol) return TElemento;
	-- Devuelve la informacin asociada a la raz de T.
	-- T no puede ser un rbol vaco
    
	function Izquierdo(T: rbol) return rbol;
	-- Devuelve el subrbol izquierdo de T.
	-- T no puede ser un rbol vaco
    
	function Derecho(T: rbol) return rbol;
	-- Devuelve el subrbol derecho de T.
	-- T no puede ser un rbol vaco

	function Copia(T: rbol) return rbol;
	-- Devuelve una copia de T

	rbol_Vaco: exception;
	-- Se lanza cuando se intenta realizar una operacin
	-- no aplicable a los rboles vacos

private
	-- Estructura de datos del tipo rbol
	type Nodo;
	type rbol is access Nodo;
	type Lista_Hijos is array(1..2) of rbol;
	type Nodo is record
		Info : TElemento;   -- Informacin del nodo
		Hijos: Lista_Hijos; -- Hijos(1) = hijo izquierdo
						  	-- Hijos(2) = hijo derecho
	end record;

	-- Definicin del rbol vaco
	Vaco : constant rbol := null;
end rbol_Bin;
