package montculos;
import java.util.ArrayList;
/**
 * Implementacin de una cola de prioridad usando un montculo
 */
public class ColaPrioridad 
{
	ArrayList montculo;  // Lista de elementos de la cola
	int numElementos = 0; // Tamao actual de la cola

	/**
	 * Constructor por defecto con capacidad inicial para un rbol
	 * con cuatro niveles
	 */
	public ColaPrioridad() 
	{
		montculo = new ArrayList(15);
	}

	/**
	 * Constructor con capacidad inicial elegible
	 */
	public ColaPrioridad(int capacidadInicial) 
	{
		montculo = new ArrayList(capacidadInicial);
	}

	/**
	 * Intercambia los valores de sus parmetros (x, y)
	 */
	private void intercambiar(int x, int y) 
	{
		Object z = montculo.get(x);
		montculo.set(x, montculo.get(y));
		montculo.set(y, z);
	}

	/**
	 * Hace ascender un elemento por un montculo hasta que
	 * alcance la posicin que le corresponde segn su prioridad
	 */
	private void flotar(int p) 
	{
		// Si el elemento en p no es la raz, se compara con su padre
		// para ver quin tiene mayor prioridad
		if ((p > 0)
			&& (((Comparable) montculo.get(p))
			.compareTo((Comparable) montculo.get((p - 1) / 2)))
			> 0) 
		{
			// Se intercambia y se repite la operacin de flotar
			intercambiar(p, (p - 1) / 2);
			flotar((p - 1) / 2);
		}
	}

	/**
	 *  Aade un elemento al final del montculo y llama
	 * a flotar para hacerlo llegar a su posicin
	 */
	public void insertar(Comparable e) 
	{
		if (numElementos == montculo.size())
			montculo.ensureCapacity(montculo.size() * 2 + 1);

		montculo.add(numElementos++, e);
		flotar(numElementos - 1);
	}

	/**
	 * Hace ascender un elemento por un montculo hasta que
	 * alcance la posicin que le corresponde segn su prioridad
	 */
	private void hundir(int p) 
	{
		int izq = 2 * p + 1;
		int der = 2 * p + 2;
		if (izq < numElementos) 
		{ // Tiene hijos
			if ((der < numElementos)
				&& (((Comparable) montculo.get(der))
				.compareTo((Comparable) montculo.get(izq)))
				> 0) 
			{
				// Tiene ambos hijos y el derecho es el de mayor prioridad
				if (((Comparable) montculo.get(der)).compareTo(
					(Comparable) montculo.get(p))
					> 0) 
				{
					// La prioridad del hijo derecho es mayor que la de p
					intercambiar(p, der);
					hundir(der);
				}
			}
			else 
			{
				// slo tiene un hijo izquierdo o ste es de mayor prioridad
				if (((Comparable) montculo.get(izq)).compareTo(
					(Comparable) montculo.get(p))
					> 0) 
				{
					// La prioridad del hijo izquierdo es mayor que la de p
					intercambiar(p, izq);
					hundir(izq);
				}			
			}
		}
	}

	/**
	 * Vaca la cola de prioridad
	 */
	public void vaciar() 
	{
		montculo = null;
		numElementos = 0;	
	}

	/**
	 * Quita el ltimo elemento, lo pone en lugar de la raz
	 * y llama a hundir para hacerlo llegar a su posicin
	 */
	public void extraer() 
	{
		montculo.set(0, montculo.get(--numElementos));
		hundir(0);
	}
	
	/**
	 * @return el elemento de mayor prioridad
	 */
	public Comparable examinar() 
	{
		return (Comparable) montculo.get(0);
	}
	
	/**
	 * @return el nmero de elementos de la cola de prioridad
	 */
	public int tamao() 
	{
		return numElementos;
	}
}