package tries;

import contenedores.Contenedor;

/**
 * Implementacin de un contenedor usando una estructura interna de rbol trie
 */
public class Trie implements Contenedor {
	private class NodoTrie {
		public NodoTrie insertar(Object e,  String clave, int p) {
			if (this instanceof NodoDesviacin) {
				NodoDesviacin aux = (NodoDesviacin) this;
				if (clave.length() == p) {
					// Se acab la clave
					aux.fin = new NodoInformacin(e);
				}
				else {
					// Contina la insercin en el subrbol adecuado
					Integer i = new Integer(clave.substring(p, p+1));
					if (aux.enlaces[i.intValue()] == null)
						aux.enlaces[i.intValue()] = new NodoInformacin(e);
					else
					aux.enlaces[i.intValue()] = 
						aux.enlaces[i.intValue()].insertar(e, clave, p + 1);
				}
			}
			else {
				NodoInformacin aux = (NodoInformacin) this;
				if (aux.elemento.hashCode() != e.hashCode()) {
					// Se crea un nodo desviacin para separar la 
					// clave existente y la nueva
					NodoDesviacin nuevo = new NodoDesviacin();
					Integer claveAux;
					if (aux.elemento.hashCode() >= 0)
						claveAux = new Integer(aux.elemento.hashCode());
					else claveAux = new Integer(-aux.elemento.hashCode());
					if (p >= claveAux.toString().length())
						nuevo.fin = this;
					else {
						Integer i =
							new Integer(claveAux.toString().substring(p, p+1));
						nuevo.enlaces[i.intValue()] = this;
					}
					return nuevo.insertar(e, clave, p);	
				}
			}
			return this;
		}
		
		public NodoTrie extraer(Object e,  String clave, int p) {
			if (this instanceof NodoDesviacin) {
				NodoDesviacin aux = (NodoDesviacin) this;
				if (p < clave.length()) {
					// Contina la extraccin en el subrbol adecuado
					Integer i = new Integer(clave.substring(p, p+1));
					if (aux.enlaces[i.intValue()] != null)
						aux.enlaces[i.intValue()] =
							aux.enlaces[i.intValue()].extraer(e, clave, p + 1); 
				}
				else {
					// Se acab la clave
					if (aux.fin != null)
						aux.fin = aux.fin.extraer(e, clave, p + 1); 
				}

				// Se comprueba el nmero de hijo del nodo actual				
				int j = 0;
				NodoTrie noNulo = null;
				for (int i = 0; (i < 10)&&(j < 2); i++) {
					if (aux.enlaces[i] != null) {
						j++;
						noNulo = aux.enlaces[i]; 
					}
				}
				
				if (aux.fin != null) {
					j++;
					noNulo = aux.fin;
				}
				
				// Si este nodo no tiene hijos, o tiene slo uno de
				// tipo informacin, se elimina del rbol
				if ((j == 1)&&(noNulo instanceof NodoInformacin))
					// Slo tiene un hijo de tipo informacin
					return noNulo;
				else
					// No tiene hijos
					if (j == 0) return null;

			}
			else {
				// Se ha alcanzado un nodo informacin, si contiene el
				// elemento a extraer, se elimina del rbol
				NodoInformacin aux = (NodoInformacin) this;
				if (aux.elemento.hashCode() == e.hashCode()) return null;

			}
			
			// El nodo actual no se elimina
			return this;
		}
		
		public NodoTrie est(Object e,  String clave, int p) {
			if (this instanceof NodoDesviacin) {
				NodoDesviacin aux = (NodoDesviacin) this;
				if (p < clave.length()) {
					// Se contina la bsqueda en el subrbol adecuado
					Integer i = new Integer(clave.substring(p, p+1));
					if (aux.enlaces[i.intValue()] != null)
						return aux.enlaces[i.intValue()].est(e, clave, p + 1); 
				}
				else {
					// Se acab la clave, el elemento slo puede estar
					// en el enlace final
					if (aux.fin != null)
						return aux.fin.est(e, clave, p + 1);					
				}
			}
			else {
				// Si la clave del elemento alamcenado en el nodo actual
				// coincide con la del elemento buscado, entonces, es el que buscamos
				if (((NodoInformacin) this).elemento.hashCode() ==
					e.hashCode()) return this;									
			}
			
			// No se encontr
			return null;
		}
		
		public int tamao() {
			if (this instanceof NodoInformacin) return 1;
			else {
				NodoDesviacin aux = (NodoDesviacin) this;
				int numElementos = 0;
				for (int i = 0; i < 10; i++)
					if(aux.enlaces[i] != null)
						numElementos += aux.enlaces[i].tamao();
				if (aux.fin != null) numElementos++;

				return numElementos;
			}
		}
	}
	
	private class NodoInformacin extends NodoTrie {
		Object elemento;
		
		public NodoInformacin(Object e) {
			super();
			elemento = e;
		}
	}
	
	private class NodoDesviacin extends NodoTrie {
		NodoTrie enlaces[];
		NodoTrie fin;
		
		public NodoDesviacin() {
			super();
			enlaces = new NodoTrie[10];
		}
	}

	NodoTrie triePos, trieNeg;
	
	
	/* (no Javadoc)
	 * @see contenedores.Contenedor#insertar(java.lang.Object)
	 */
	public void insertar(Object e) {
		if (e.hashCode() >= 0) {
			Integer clave = new Integer(e.hashCode());
			if (triePos == null) triePos = new NodoInformacin(e);
			else triePos = triePos.insertar(e, clave.toString(), 0);
		}
		else {
			Integer clave = new Integer(-e.hashCode());
			if (trieNeg == null) trieNeg = new NodoInformacin(e);
			else trieNeg = trieNeg.insertar(e, clave.toString(), 0);			
		}
	}

	/* (no Javadoc)
	 * @see contenedores.Contenedor#extraer(java.lang.Object)
	 */
	public void extraer(Object e) {
		if (e.hashCode() >= 0)
			if (triePos != null)
				triePos = triePos.extraer(e, 
						(new Integer(e.hashCode()).toString()), 0);
		else
			if (trieNeg != null)
				trieNeg = trieNeg.extraer(e,
						(new Integer(-e.hashCode()).toString()), 0);
	}

	/* (no Javadoc)
	 * @see contenedores.Contenedor#vaciar()
	 */
	public void vaciar() {
		triePos = null;
		trieNeg = null;
	}

	/* (no Javadoc)
	 * @see contenedores.Contenedor#est(java.lang.Object)
	 */
	public boolean est(Object e) {
		if (e.hashCode() >= 0)
			if (triePos != null)
				return (triePos.est(e,
						(new Integer(e.hashCode()).toString()), 0) != null);
		else
			if (trieNeg != null)
				return (trieNeg.est(e,
						(new Integer(-e.hashCode()).toString()), 0) != null);
		
		return false;
	}

	/* (no Javadoc)
	 * @see contenedores.Contenedor#examinar(java.lang.Object)
	 */
	public Object examinar(Object e) {
		return null;
	}

	/* (no Javadoc)
	 * @see contenedores.Contenedor#esVaco()
	 */
	public boolean esVaco() {
		return ((triePos == null)&&(trieNeg == null));
	}

	/* (no Javadoc)
	 * @see contenedores.Contenedor#tamao()
	 */
	public int tamao() {
		int numElementos = 0;
		if (triePos != null) numElementos = triePos.tamao();
		if (trieNeg != null) numElementos += trieNeg.tamao();
		return numElementos;
	}
}