#ifndef INC_BINARIOBUSQUEDA
#define INC_BINARIOBUSQUEDA
#include "binbusbasico.h"
#include "nodobinario.h"

template <class T>
class BinarioBusqueda:public BinarioBusquedaBasico<T,NodoBinario<T> > {
      NodoBinario<T> *raiz;
      void insertarInterno(NodoBinario<T>*&,const T&);
      void extraerInterno(NodoBinario<T>*&,const T&);
      void extraerSucesor(NodoBinario<T>*,NodoBinario<T>*&);
   public:
      BinarioBusqueda(){raiz=NULL;}
      ~BinarioBusqueda();
      void insertar(const T&);
      void extraer(const T&);
      bool encuentra(const T&)const;
      void vaciar();
};

// Funciones miembro de la clase BinarioBusqueda
template <class T>
   BinarioBusqueda<T>::~BinarioBusqueda()
   {
      eliminarInterno(raiz);
   }

template <class T>
   void BinarioBusqueda<T>::insertarInterno(NodoBinario<T> *&nodo,const T& valor)
   {
      if(nodo==NULL)
      {
         nodo=new NodoBinario<T>(valor);
         this->numElem++;
      }
      else
         if(valor!=nodo->clave)
         {
            if(valor<nodo->clave)
               insertarInterno(nodo->enlaces[0],valor);
            else
               insertarInterno(nodo->enlaces[1],valor);
         }          
   }

template <class T>
   void BinarioBusqueda<T>::extraerSucesor(NodoBinario<T> *nodoExtraer,NodoBinario<T> *&nodo)
   {
      NodoBinario<T> *hijo;
   //nodo contiene al sucesor en simtrico del valor de clave a extraer
      if(nodo->enlaces[0]==NULL) 
      {
         hijo=nodo->enlaces[1];
         nodoExtraer->clave=nodo->clave;
         delete nodo;
         nodo=hijo;
         this->numElem--;
      }
      else
         extraerSucesor(nodoExtraer,nodo->enlaces[0]);
   }

template <class T>
   void BinarioBusqueda<T>::extraerInterno(NodoBinario<T> *&nodo,const T& valor)
   {
      int rama;
      NodoBinario<T> *hijo;
      if(nodo!=NULL)
      {
         if(valor==nodo->clave) 
         //el nodo que contiene al valor de clave a extraer tiene un hijo o no tiene ninguno
            if((nodo->enlaces[0]==NULL)||(nodo->enlaces[1]==NULL))
            {
               if(nodo->enlaces[0]==NULL)
                  rama=1;
               else
                  rama=0;
               hijo=nodo->enlaces[rama];
               delete nodo;
               nodo=hijo;
               this->numElem--;
            }
            else   //tiene dos hijos
               extraerSucesor(nodo,nodo->enlaces[1]);
         else
            if(valor<nodo->clave)
               extraerInterno(nodo->enlaces[0],valor);
            else
               extraerInterno(nodo->enlaces[1],valor);
      }       
   }

template <class T>
   bool BinarioBusqueda<T>::encuentra(const T& valor)const
   {
      return buscarInterno(raiz,valor);
   }

template <class T>
   void BinarioBusqueda<T>::insertar(const T& valor)
   {
      insertarInterno(raiz,valor);
   }

template <class T>
   void BinarioBusqueda<T>::extraer(const T& valor)
   {
      extraerInterno(raiz,valor);
   }

template <class T>
   void BinarioBusqueda<T>::vaciar()
   {
      eliminarInterno(raiz);
      raiz=NULL;
      this->numElem=0;
   }

#endif



