#ifndef INC_COLA_PRIORIDAD
#define INC_COLA_PRIORIDAD
#include <vector>
   using namespace std;
/**
 * Implementacin de una cola de prioridad usando un montculo
 */
template <class T>
class ColaPrioridad  {
   public: 
   // Constructoras
      ColaPrioridad(){};
      ~ColaPrioridad(){};
   
      /**
       * Aade un elemento al final del montculo y llama
       * a flotar para hacerlo llegar a su posicin
       */
      void insertar(const T &e) {
         elementos.push_back(e);
         flotar(elementos.size() - 1);
      }   
   
      /**
       * Quita el ltimo elemento, lo pone en lugar de la raz
       * y llama a hundir para hacerlo llegar a su posicin
       */
      void extraer() {
         elementos[0] = elementos[elementos.size()-1];
         elementos.pop_back();
         hundir(0);
      }
   
      /**
       * Vaca la cola de prioridad
       */
      void vaciar() {
         elementos.clear();
      }
   
   // Observadoras
      /**
       * Devuelve el elemento de mayor prioridad
       */
      T examinar() const {
         return elementos[0];
      }
   
      /**
       * Devuelve el nmero de elementos de la cola de prioridad
       */
      int cardinal() const {
         return elementos.size();
      }
   
   private:
      vector<T> elementos;
   
      /**
       * Intercambia los valores de sus parmetros (x, y)
       */
      void intercambiar(int x, int y) {
         T z = elementos[x];
         elementos[x] = elementos[y];
         elementos[y] = z;
      }
   
      /**
       * Hace ascender un elemento por un montculo hasta que
       * alcance la posicin que le corresponde segn su prioridad
       */
      void flotar(int p) {
      // Si el elemento en p no es la raz, se compara con su padre
      // para ver quin tiene mayor prioridad
         if ((p > 0) && (elementos[p] > elementos[(p - 1) / 2])) {
         // Se intercambia y se repite la operacin de flotar
            intercambiar(p, (p - 1) / 2);
            flotar((p - 1) / 2);
         }
      }
   
      /**
       * Hundir hace descender un elemento por un montculo hasta que
       * alcance la posicin que le corresponde segn su prioridad
       */
      void hundir(int p) {
         unsigned int izq = 2 * p + 1;
         unsigned int der = 2 * p + 2;
         if (izq < elementos.size()) { // Tiene hijos
            if ((der < elementos.size()) && 
            	(elementos[der] > elementos[izq])) {
               // Tiene ambos hijos y el derecho es el de mayor prioridad
               if (elementos[der] > elementos[p]) {
                  // La prioridad del hijo derecho es mayor que la de p
                  intercambiar(p, der);
                  hundir(der);
               }
            }
            else {
               // Slo tiene un hijo izquierdo o ste es de mayor prioridad
               if (elementos[izq] > elementos[p]) {
                  // La prioridad del hijo izquierdo es mayor que la de p
                  intercambiar(p, izq);
                  hundir(izq);
               }			
            }
         }
      }
   
};

#endif



