#ifndef INC_TRIE
#define INC_TRIE
#include "..\comun\contenedor.h"
#include <string>
#include <cstring>
#include <cstdlib>
#include <iostream>
   using namespace std;

template <class T>
class Trie:public Contenedor<T>  {
   public: 
   // Constructoras
      Trie(){
         trie = NULL;
      };
      ~Trie(){};
   
      void insertar(const T &);
      void extraer(const T &);
      void vaciar();
   
   // Observadoras
      bool encuentra(const T &) const;
      int cardinal() const;
   
   private:
      // Clase del nodo
      typedef enum {DESV,INFO} TipoNodo;
   
      struct NodoTrie {
         TipoNodo tipo;
      
         NodoTrie(TipoNodo tn) {
            tipo = tn;
         }
      
         NodoTrie *insertar(T e,  string cl, int p) {
            if (tipo == DESV){
               NodoDesv *aux =  (NodoDesv *) this;
               if (cl.length() == (unsigned int)p) {
                  // Se acab la clave
                  aux->fin = new NodoInfo(e);
               }
               else {
                  // Contina la insercin en el subrbol adecuado
                  int i = atoi(cl.substr(p,1).c_str());
                  if (aux->enlaces[i] == NULL)
                     aux->enlaces[i] = new NodoInfo(e);
                  else
                     aux->enlaces[i] = aux->enlaces[i]->insertar(e, cl, p+1);
               }
            }
            else {
               NodoInfo *aux =  (NodoInfo *) this;
               if (clave(aux->elemento) != clave(e)) {
                  // Se crea un nodo desviacin para separar la 
                  // clave existente y la nueva
                  NodoDesv *nuevo = new NodoDesv();
                  unsigned int claveAux = clave(aux->elemento);
                  char buffer[10];
                  sprintf (buffer, "%d", claveAux);
                  string clAux(buffer);
                  if ((unsigned int)p >= clAux.length())
                     nuevo->fin = this;
                  else {
                     int i = atoi(clAux.substr(p,1).c_str());
                     nuevo->enlaces[i] = this;
                  }
                  return nuevo->insertar(e, cl, p);	
               }
            }
            return this;
         };
      
      
         NodoTrie *extraer(T e,  string cl, int p) {
            if (tipo == DESV) {
               NodoDesv *aux = (NodoDesv *) this;
               if ((unsigned int)p < cl.length()) {
                  // Contina la extraccin en el subrbol adecuado
                  int i = atoi(cl.substr(p,1).c_str());
                  if (aux->enlaces[i] != NULL){
                  	 // Se guarda la direccin del nodo por si debe
                  	 // eliminarse
                     NodoTrie *antiguo = aux->enlaces[i];
                     aux->enlaces[i] = 
                        aux->enlaces[i]->extraer(e, cl, p+1);
                     // Si el nodo devuelto es distinto del antiguo, es
                     // que el antiguo debe eliminarse
                     if (aux->enlaces[i] != antiguo) delete antiguo;
                  }
               }
               else {
                  // Se acab la clave
                  if (aux->fin != NULL) {
                     NodoTrie *antiguo = aux->fin;
                     aux->fin = aux->fin->extraer(e, cl, p + 1); 
                     if (aux->fin != antiguo) delete antiguo;
                  }
               }
            
               // Se comprueba el nmero de hijos del nodo actual				
               int j = 0;
               NodoTrie *noNulo = NULL;
               int posNoNulo;
               for (int i = 0; (i < 10)&&(j < 2); i++) {
                  if (aux->enlaces[i] != NULL) {
                     j++;
                     posNoNulo = i;
                     noNulo = aux->enlaces[i]; 
                  }
               }
            
               if (aux->fin != NULL) {
                  j++;
                  posNoNulo = -1;
                  noNulo = aux->fin;
               }
            
               // Si este nodo no tiene hijos, o tiene slo uno de
               // tipo informacin, se elimina del rbol
               if ((j == 1)&&(noNulo->tipo == INFO)) {
                  // Slo tiene un hijo de tipo informacin
                  // Si posNoNulo es negativo, es aux->fin
                  if (posNoNulo >= 0) aux->enlaces[posNoNulo] = NULL;
                  else aux->fin = NULL;
                  return noNulo;
               }
               else
                  // No tiene hijos
                  if (j == 0) 
                     return NULL;
            
            }
            else {
               // Se ha alcanzado un nodo informacin, si contiene el
               // elemento a extraer, se elimina del rbol
               NodoInfo *aux = (NodoInfo *) this;
               if (clave(aux->elemento) == clave(e)) 
                  return NULL;
            }
         
            // El nodo actual no se elimina
            return this;
         }
      
      
         NodoTrie *encuentra(T e,  string cl, int p) {
            if (tipo == DESV) {
               NodoDesv *aux = (NodoDesv *) this;
               if ((unsigned int)p < cl.length()) {
                  // Se contina la bsqueda en el subrbol adecuado
                  int i = atoi(cl.substr(p,1).c_str());
                  if (aux->enlaces[i] != NULL)
                     return aux->enlaces[i]->encuentra(e, cl, p+1); 
               }
               else {
                  // Se acab la clave, el elemento slo puede estar
                  // en el enlace final
                  if (aux->fin != NULL)
                     return aux->fin->encuentra(e, cl, p + 1);		
               }
            }
            else {
               // Si la clave del elemento alamcenado en el nodo actual
               // coincide con la del elemento buscado, entonces, es el
               // que buscamos
               NodoInfo *aux = (NodoInfo *) this;
               if (clave(aux->elemento) == clave(e)) 
                  return this;					
            }
         
            // No se encontr
            return NULL;
         }
      
         int cardinal() {
            if (tipo == INFO) 
               return 1;
            else {
               NodoDesv *aux = (NodoDesv *) this;
               int numElementos = 0;
               for (int i = 0; i < 10; i++)
                  if(aux->enlaces[i] != NULL) numElementos += aux->enlaces[i]->cardinal();
               if (aux->fin != NULL) numElementos++;
            
               return numElementos;
            }
         }
      
      };
   
      struct NodoInfo: public NodoTrie {
         T elemento;
      
         NodoInfo(T e):NodoTrie(INFO) {
            elemento = e;
         }
      };
   
      struct NodoDesv: public NodoTrie {
         NodoTrie *enlaces[10];
         NodoTrie *fin;
         NodoDesv():NodoTrie(DESV) {
            fin = NULL;
            for (int i = 0; i < 10; i++) enlaces[i] = NULL;
         }
         ~NodoDesv() {
            for (int i = 0; i < 10; i++)
               if (enlaces[i] != NULL) delete enlaces[i];
            if (fin != NULL) delete fin;
         }
      
      };
   
      NodoTrie *trie;
   
   };

/**
 * Inserta un elemento en el contenedor, si est no hace nada. Lanza una
 * excepcin si no puede insertar
 */
template <class T>
void Trie<T>::insertar(const T &e) {
   unsigned int cl = clave(e);
   if (trie == NULL) 
      trie = new NodoInfo(e);
   else {
      char buffer[10];
      sprintf (buffer, "%d", cl);
      string s(buffer);
      trie = trie->insertar(e,s,0);
   }
};

template <class T>
void Trie<T>::extraer(const T &e) {
   if (trie != NULL) {
      unsigned int cl = clave(e);
      char buffer[10];
      sprintf (buffer, "%d", cl);
      string s(buffer);
      // Se guarda la direccin de la raz por si cambia
      // y hay que eliminar el nodo original
      NodoTrie *aux = trie;
      trie = trie->extraer(e,s,0);
      if (trie != aux) delete aux;
   }
};
template <class T>
void Trie<T>::vaciar() {
   if (trie != NULL) delete trie;
   trie = NULL;
};

template <class T>
bool Trie<T>::encuentra(const T &e) const {
   if (trie != NULL) {
      unsigned int cl = clave(e);
      char buffer[10];
      sprintf (buffer, "%d", cl);
      string s(buffer);
      return (trie->encuentra(e,s,0) != NULL);
   }
   else 
      return false;
};

template <class T>
int Trie<T>::cardinal() const{
   if (trie != NULL) {
      return trie->cardinal();
   }
   else 
      return 0;
};


#endif



